package de.wwwu.awolf.presenter.algorithms.naiv;

import de.wwwu.awolf.model.Line;
import de.wwwu.awolf.model.Point;
import de.wwwu.awolf.presenter.algorithms.Algorithm;
import de.wwwu.awolf.presenter.util.FastElementSelector;
import de.wwwu.awolf.presenter.util.Logging;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Implementierung verschiedener Algorithmen zur Berechnung von Ausgleichsgeraden.
 *
 * @Author: Armin Wolf
 * @Email: a_wolf28@uni-muenster.de
 * @Date: 15.09.2017.
 */
public class NaivLeastMedianOfSquaresEstimator implements Algorithm {
    private List<Point> set = new ArrayList<>();

    private int n;
    private double ds, b, m;

    /**
     * Konstruktor
     *
     * @param lines Liste des Geraden
     */
    public NaivLeastMedianOfSquaresEstimator(List<Line> lines) {
        for (Line l : lines) {
            set.add(new Point(l.getM(), l.getB()));
        }
    }

    /**
     * Crude Algorithmus zum berechnen des LSM-Schätzers.
     */
    private Line crudeAlg() {
        ds = Double.MAX_VALUE;
        b = 0d;
        m = 0d;
        List<Point> triple = new ArrayList<>();
        double beta;
        double alpha;
        double dijk;
        Logging.logInfo("=== S T A R T - naiv L M S ===");
        long start;
        long end;
        start = System.currentTimeMillis();
        for (Point i : set) {
            for (Point j : set) {
                for (Point k : set) {
                    triple.add(i);
                    triple.add(j);
                    triple.add(k);
                    Collections.sort(triple);
                    beta = (triple.get(0).getY() - triple.get(2).getY()) / (triple.get(0).getX() - triple.get(2).getX());
                    alpha = (triple.get(1).getY() + triple.get(2).getY() - (beta * (triple.get(1).getX() + triple.get(2).getX()))) * 0.5;
                    dijk = f(alpha, beta);
                    if (dijk < ds) {
                        ds = dijk;
                        b = alpha;
                        m = beta;
                    }
                    triple.clear();
                }
            }
        }
        end = System.currentTimeMillis();
        Logging.logInfo("Zeit: " + ((end - start) / 1000));

        return new Line(getSlope(), getYInterception());
    }

    /**
     * Berechnet die Gerade mit dem medianen Fehler
     *
     * @param a y-Achsenabschnitt
     * @param b Steigung
     * @return medianer Fehler
     */
    private Double f(double a, double b) {
        List<Double> res = new ArrayList<>();
        for (Point p : set) {
            res.add(Math.abs(p.getY() - (a + b * p.getX())));
        }
        return FastElementSelector.randomizedSelect(res, res.size() * 0.5);
    }

    @Override
    public Line call() {
        return crudeAlg();
    }

    /**
     * @return y-Achsenabschnitt
     */
    public Double getYInterception() {
        return b * -1;
    }

    /**
     * @return Steigung
     */
    public Double getSlope() {
        return m * -1;
    }
}
